# -*-coding:utf-8-*-
# !/usr/bin/python
# coding: iso-8859-15
from common_utils import update
from common_utils import manager
from common_utils import Remote
import collections
import os


def check_heartbeat_config():
    filename = "/etc/corosync/corosync.conf"
    if not os.path.exists(filename):
        return False
    return True


def get_crm_metadata(crm_cmd):

    if crm_cmd in ["pengine", "crmd"]:
        metadata = manager.get_crm_metadata(crm_cmd)
    else:
        return {"action": False, "error": _("input error, the cmd must be pengine or crmd")}
    name = metadata.name
    version = metadata.version
    longdesc = metadata.longdesc
    shortdesc = metadata.shortdesc
    parameters = metadata.parameters
    # 节点数
    nodecount = len(manager.get_all_nodes())
    # 心跳是否已经配置
    isconfig = check_heartbeat_config()

    crm_config = get_crm_config(metadata)
    parameters_value = {}
   
    # 调试param返回值 
    for param in parameters:
        config = {}
        config = param
        config['value'] = crm_config[param['name']]
        parameters_value[param['name']] = config

    # 对首选项数据进行排序
    sort_parameters_value = collections.OrderedDict()
    for param in  ['symmetric-cluster', 'no-quorum-policy', 'stonith-enabled', 'start-failure-is-fatal', 'maintenance-mode',
                       'default-resource-stickiness',  'node-health-strategy', 'node-health-red', 'node-health-yellow', 'node-health-green']:
	sort_parameters_value[param] = parameters_value[param]

 
    return {"action": True,
            'data': {"isconfig": isconfig, "nodecount": nodecount, "name": name, "version": version,
                     "longdesc": longdesc, "shortdesc": shortdesc, "parameters": sort_parameters_value}}


def get_crm_config(metadata):
    global mode_level
    config = {}
    for parameter in metadata.parameters:
        config[parameter["name"]] = str(parameter["content"]["default"])

    cluster_property_sets = manager.xml_nodes["cib"].getElementsByTagName("cluster_property_set")
    if len(cluster_property_sets) == 0:
        return config
    elif len(cluster_property_sets) == 1:
        for nvpair in cluster_property_sets[0].getElementsByTagName("nvpair"):
            name = nvpair.getAttribute("name")
            value = nvpair.getAttribute("value")
            if value != "":
                config[nvpair.getAttribute("name")] = nvpair.getAttribute("value")
        return config
    else:
        return None


def save_crm_metadata(data):

    attribute = data
    for name in attribute.keys():
        type = 'crm_config'
        value = attribute[name]
        manager.set_crm_attribute(type, str(name), str(value))
        if manager.failed_reason != "":
            return {'action': False, 'error': _(manager.failed_reason)}
    update()
    return {'action': True, 'info': _("Save crm metadata success")}


def cluster_action(nodeauth, action):
    if 'nodeauth' in nodeauth:
        for node in nodeauth['nodeauth']:
            nodeip = node['nodeip']
            password = node['password']
            remote = Remote(nodeip, password)
            if action == 'start':
                remote.service_start()
            if action == 'stop':
                remote.service_stop()
            if action == 'restart':
                remote.service_restart()
            return {'action': True, 'info': _("Action on cluster success")}

    return {'action': True, 'error': _("Cluster need auth info")}


if __name__ == '__main__':
    # print get_crm_metadata("pengine")

    nodeauth = {
        "nodeauth": [
            {
                "nodeid": "ns187",
                "nodeip": "10.1.110.187",
                "password": "qwer1234"
            },
            {
                "nodeid": "ns188",
                "nodeip": "10.1.110.188",
                "password": "qwer1234"
            }
        ]
    }

    cluster_action(nodeauth, "start")
