#!/usr/bin/python
# -*-coding:utf-8-*-
# coding: iso-8859-15

from bottle import Bottle, run, request, static_file, redirect, response

import sys
import locale
import gettext
import json

sys.path.append("/usr/share/heartbeat-gui")
sys.path.append("/usr/lib64/heartbeat-gui")

import resource
import meta
import rsc_thread
import cluster
import node
import commands
import alarm
import common_utils
from common_utils import AUTH_USER, AUTH_PASSWORD
import config
import ha_license
from datetime import datetime
from functools import wraps
import logging
import drbd
import gfs
import log
import scripts

# from beaker.middleware import SessionMiddleware


app_name = "ha-api"
gettext.bindtextdomain(app_name)
gettext.textdomain(app_name)
gettext.install(app_name, "/usr/share/locale", unicode=1)
gettext.translation(app_name, "/usr/share/locale", languages=["zh_CN"]).install(True)

# 配置日志功能
logger = logging.getLogger('ha-api')
# set up the logger
logger.setLevel(logging.INFO)
file_handler = logging.FileHandler('/var/log/ha-api.log')
formatter = logging.Formatter('%(msg)s')
file_handler.setLevel(logging.DEBUG)
file_handler.setFormatter(formatter)
logger.addHandler(file_handler)


# 设置日志返回信息
def log_to_logger(fn):
    '''
    Wrap a Bottle request so that a log line is emitted after it's handled.
    (This decorator can be extended to take the desired logger as a param.)
    '''
    @wraps(fn)
    def _log_to_logger(*args, **kwargs):
        request_time = datetime.now()
        actual_response = fn(*args, **kwargs)
        # modify this to log exactly what you need:
        logger.info('%s %s %s %s %s' % (request.remote_addr,
                                        request_time,
                                        request.method,
                                        request.url,
                                        response.status))
        return actual_response
    return _log_to_logger


app = Bottle()
app.install(log_to_logger)

# if not common_utils.is_ha_login():
#    print "Please ensure that the HA service has already been started, you can run 'systemctl start neokylinha' to start HA service!"
#    exit(0)

@app.route('/api/v1/login', method='POST')
def check_auth():
    data = request.json
    if data is not None and 'username' in data and 'password' in data:
        username = data['username']
	if username !=  'hacluster':
	    return {'action': False, 'error': _('Username is not allowed to login')}
        password = data['password']
        ret = common_utils.check_auth(username, password)
        if ret:
            # 更新 session 信息
            session = request.environ.get('beaker.session')
            session['user_name'] = username
            session['password'] = password
            session.save()
            return {'action': ret, 'info': _('Login success')}

        return {'action': ret, 'error': _('Username or password error')}

    return {'action': False, 'error': _('Login failed')}


@app.route('/api/v1/logout', method='POST')
def logout():
    session = request.environ.get('beaker.session')
    session.delete()
    return {'action': True, 'info': _('Login failed')}


@app.route('/api/v1/haclusters/1/resources/meta_attributes/:category', method='GET')
def get_rsc_meta_attributes(category):
    if category not in ["clone", "primitive", "group"]:
        return {'action': False, 'error': _('The category is not allowed')}

    return resource.get_rsc_meta_attributes(category)


@app.route('/api/v1/haclusters/1/nodes/:nodeid/:action', method='PUT')
def node_action(nodeid, action):
    action_list = ["start", "stop", "restart", "standby", "unstandby"]
    if action not in action_list:
        return {'action': False, 'error': _('The node action is not allowed')}

    if action in ["start", "stop", "restart"]:
        return node.node_action(nodeid, action)
    # standby or unstandby
    return node.set_node_action(nodeid, action)

@app.route('/api/v1/haclusters/1/localnodes/:action', method='PUT')
def node_local_action(action):
    if action in ["start", "stop", "restart"]:
        return node.node_local_action(action)
    return {'action': False, 'error': _('The node action is not allowed')}

@app.route('/api/v1/haclusters/1/localnodes/:action', method='POST')
def node_local_action_post(action):
    if action in ["start", "stop", "restart"]:
        return node.node_local_action(action)
    return {'action': False, 'error': _('The node action is not allowed')}


@app.route('/api/v1/haclusters/1', method='PUT')
def change_crm_metadata():
    return cluster.save_crm_metadata(request.json)


@app.route('/api/v1/haclusters/1/:action', method='PUT')
def cluster_action(action):
    return cluster.cluster_action(request.json, action)


@app.route('/api/v1/haclusters/1/resources/:rsc_id', method='PUT')
def set_rsc_info(rsc_id):
    data = request.json
    # 将获得的bool类型、整形数据转化为字符串
    data = resource.data_int_bool_to_str(data)
    return resource.set_rsc_info(data, rsc_id)


@app.route('/api/v1/haclusters/1/resources/:rsc_id/:action', method='PUT')
def action_on_rsc1(rsc_id, action):
    data = request.json
    action_list = ['start', 'stop', 'location', 'colocation', 'order', 'cleanup', 'migrate', 'unmigrate', 'delete']

    if action in action_list:
        if action in ['start', 'stop']:
            return resource.action_on_rsc(rsc_id, action)
        if action == "location":
            return resource.rsc_run_level(data, rsc_id)
        if action == "colocation":
            return resource.set_colocation(data, rsc_id)
        if action == "order":
            return resource.set_order(data, rsc_id)
        if action == "cleanup":
            return resource.cleanup_rsc(rsc_id)
        if action == "migrate":
            return resource.migrate_rsc(data, rsc_id)
        if action == "unmigrate":
            return resource.unmigrate_rsc(rsc_id)
        if action == "delete":
            return resource.delete_rsc(rsc_id)

    return {'action': False, 'error': _('The resource action is not allowed')}


@app.route("/api/v1/haclusters/1/resources/:rsc_id/relations/:relation", method='GET')
def get_localtion(rsc_id, relation):
    if relation == "location":
        return resource.get_run_level(rsc_id)
    if relation == "colocation":
        return resource.get_colocation(rsc_id)
    if relation == "order":
        return resource.get_order(rsc_id)
    return {'action': False, 'error': 'The relation is not allowed'}


@app.route('/api/v1/haclusters/1', method='GET')
def get_cluster_info():
    return cluster.get_crm_metadata("pengine")


@app.route("/api/v1/haclusters/1/metas", method='GET')
def get_all_meta():
    return meta.get_all_rsc_avail()


@app.route('/api/v1/haclusters/1/nodes', method='GET')
def get_all_node_list():
    return node.get_all_node_status()


@app.route('/api/v1/haclusters/1/nodes/:nodeid', method='GET')
def get_node_ip(nodeid):
    return node.get_node_ip(nodeid)


@app.route('/api/v1/haclusters/1/resources', method='GET')
def get_all_resources():
    return resource.get_all_rsc_status_constraints()

@app.route('/api/v1/haclusters/1/resources/:rsc_id', method='GET')
def get_rsc_info(rsc_id):
    ret = resource.single_rsc_get_info(rsc_id)
    if 'action' in ret and not ret['action']:
        return ret

    result = ret
    data = result['data']
    if 'primitive' in data:
        result['data'] = resource.rsc_primitive_format(data)
    if 'group' in data:
        result['data'] = resource.rsc_group_format(data)
    if 'clone' in data:
        result['data'] = resource.rsc_clone_format(data)
    return result


@app.route('/api/v1/haclusters/1/alarms', method='GET')
def get_alarm():
    return alarm.alarm_get()


@app.route('/api/v1/haclusters/1/alarms', method='POST')
def set_alarm():
    data = request.json
    return alarm.alarm_set(data)


@app.route('/api/v1/haclusters/1/alarms', method='DELETE')
def delete_alarm():
    return alarm.alarm_delete()

@app.route('/api/v1/haclusters/1/configs', method='GET')
def get_hb_configs():
    # 标识是否获取磁盘心跳信息
    get_diskhb = True
    return config.get_hb_config_org(get_diskhb)

@app.route('/api/v1/haclusters/1/get_diskhb_device', method='GET')
def get_diskhb_device():
    return config.get_diskhb_device()

@app.route('/api/v1/haclusters/1/configs', method='POST')
def set_hb_configs():
    data = request.json
    return config.save_hb_conf(data)


@app.route('/api/v1/haclusters/1/metas/:rsc_class/:rsc_type', method='GET')
@app.route('/api/v1/haclusters/1/metas/:rsc_class/:rsc_type/:rsc_provider', method='GET')
def get_rsc_meta(rsc_class, rsc_type, rsc_provider=None):
    return meta.get_rsc_meta(rsc_class, rsc_type, rsc_provider)


@app.route('/api/v1/haclusters/1/resources', method='POST')
def add_rsc():
    data = request.json
    category = data['category']
    # 将获得的bool类型、整形数据转化为字符串
    data = resource.data_int_bool_to_str(data)
    data.pop('category')
    if category == "primitive":
        parent_type = "resources"
        return resource.add_primitive_rsc(data, parent_type)
    if category == "clone":
        return resource.rsc_to_clone(data)
    if category == "group":
        return resource.rscs_to_group(data)

    return {'action': False, 'error': _('The category is not allowed')}


@app.route('/api/v1/haclusters/1/rlicenses', method='POST')
def get_remote_license_info():
    data = request.json
    return ha_license.get_remote_ha_host_license(data)


@app.route('/api/v1/haclusters/1/licenses', method='GET')
def get_license_info():
    return ha_license.get_ha_host_license()


@app.route('/api/v1/haclusters/1/licenses', method='POST')
def install_license():
    data = request.json
    return ha_license.install_ha_host_license_own(data)


@app.route('/api/v1/haclusters/1/drbd', method='GET')
def get_drbd():
    return drbd.get_drbd()

@app.route('/api/v1/haclusters/1/drbd', method='POST')
def set_drbd():
    data = request.json
    return drbd.set_drbd(data)

@app.route('/api/v1/haclusters/1/remotedrbd', method='POST')
def set_drbd():
    data = request.json
    return drbd.save_local_drbd(data)

@app.route('/api/v1/haclusters/1/drbd', method='DELETE')
def del_drbd():
    return drbd.del_drbd()

@app.route('/api/v1/haclusters/1/drbd_status', method='GET')
def get_drbd_status():
    return drbd.get_drbd_status()

@app.route('/api/v1/haclusters/1/gfs', method='GET')
def get_gfs():
    return gfs.get_gfs()

@app.route('/api/v1/haclusters/1/gfs', method='POST')
def set_gfs():
    data = request.json
    return gfs.set_gfs(data)

@app.route('/api/v1/haclusters/1/gfs', method='DELETE')
def del_gfs():
    return gfs.del_gfs()

@app.route('/api/v1/haclusters/1/hbstatus', method='GET')
def get_hb_status():
    return config.get_hb_status()

# 判断远程api是否可连接接口
@app.route('/api/v1/haclusters/1/apistatus', method='GET')
def get_api_status():
    return {'action': True}

# 远程调用保存脚本
@app.route('/api/v1/haclusters/1/remoteconfigs', method='POST')
def save_remote_config():
    data = request.json
    return config.save_local_config(data)


@app.route('/api/v1/haclusters/1/logs', method='GET')
def generate_log():
    return log.generate_log()

@app.route('/api/v1/haclusters/1/scripts', method='POST')
def generate_scripts():
    data = request.json
    return scripts.generate_scripts(data)

@app.route('/api/v1/haclusters/1/remotescripts', method='POST')
def generate_scripts():
    data = request.json
    return scripts.generate_local_scripts(data)


@app.route('/api/v1/haclusters/1/commands/:cmd_type', method='GET')
def get_commands_result(cmd_type):
    command_type = int(cmd_type)    
    if not isinstance(command_type, int):
	{'action': False, 'error': _("Please check input commands type")}
	
    command_table = {0: "All", 1: "crm_mon -1 -o", 2: "crm_simulate -Ls", 3: "pcs config show", 4: "corosync-cfgtool -s", 5: "crm configure verify"}
    if command_type != 0 and command_type in command_table:
	command_str = command_table[command_type]
	status, output = commands.getstatusoutput(command_str)
	return {'action': True, 'data': output}
    else:
	return {'action': False, 'error': _("Please check input commands type")}
	

@app.route('/api/v1/haclusters/1/commands', method='GET')
def get_commands_list():
    command_table = {1: "crm_mon -1 -o", 2: "crm_simulate -Ls", 3: "pcs config show", 4: "corosync-cfgtool -s", 5: "crm configure verify"}
    return {'action': True, 'data': command_table}


@app.route('/', method='GET')
def send_index_html():
    """
    url mapping for index html file, which with @login_required decorated
    """
    # 检查是否已经登录，如果已登录，则继续；如果未登录则检测URL 地址
    session = request.environ.get('beaker.session')
    user_name = session.get('user_name')
    if user_name:
        return static_file('index.html', root='/usr/share/heartbeat-gui/ha-api/static/')
    redirect('/login')


@app.route('/<filename:re:.+\.(html|js|css|png|woff|ttf|ico|tar)>')
def send_html_static(filename):
    """
    url mapping for html files, which also with @login_required decorated
    """
    return static_file(filename, root='/usr/share/heartbeat-gui/ha-api/static')


# Add graceful shutdown by ctrl-c signal
import signal


def app_exit(signalnum, frame):
    sys.exit(1)


signal.signal(signal.SIGINT, app_exit)


@app.error(404)
def err_404_handler(err):
    if request.is_ajax:
        return {'action': False, 'error': u'您访问的接口不存在'}
    return static_file('index.html', root='/usr/share/heartbeat-gui/ha-api/static/')


from bottle import response, HTTPResponse


@app.hook('before_request')
def login_check():
    #print 'run login_check() before request'
    # 检查是否已经登录，如果已登录，则继续；如果未登录则检测URL 地址
    session = request.environ.get('beaker.session')
    user_name = session.get('user_name')
    user, pwd = None, None

    # 系统的内部接口
    if request.path.startswith('/api/v1/haclusters/1/remoteconfigs') and request.method == 'POST' \
        or request.path.startswith('/api/v1/haclusters/1/localnodes') and request.method == 'POST' \
	or request.path.startswith('/api/v1/haclusters/1/remotescripts') and request.method == 'POST' \
	or request.path.startswith('/api/v1/haclusters/1/remotedrbd') and request.method == 'POST' \
        or request.path.startswith('/api/v1/haclusters/1/apistatus') and request.method == 'GET': 
        if request.auth is not None:
	    user, pwd = request.auth
	    if user != AUTH_USER and user != AUTH_PASSWORD:
	        raise HTTPResponse(status=403)
	else:
	    raise HTTPResponse(status=403)

    else:
        if user_name is None:
            if not request.path.startswith('/api/v1/login'):
                if request.path.startswith('/api'):
	            raise HTTPResponse(status=403)
        else:
            user_name = str(user_name).strip()
            password = str(session.get('password')).strip()
            # 这两个接口必须允许执行,否认无法登录或者启动服务
            if not request.path.startswith('/api/v1/login') and not request.path.startswith('/api/v1/haclusters/1/localnodes') and \
                not request.path.startswith('/api/v1/haclusters/1/licenses') and not request.path.startswith('/api/v1/logout') and \
                not request.path.startswith('/api/v1/haclusters/1/configs') and not request.path.startswith('/api/v1/haclusters/1/get_diskhb_device'):
                if request.path.startswith('/api'):
		    ret = common_utils.is_ha_login("127.0.0.1", user_name, password)
                    if not ret:
                        raise HTTPResponse(status=200, header={'Cache-Control': 'no-store'}, output=json.dumps({'action': False, 'error': _('Please ensure that the HA service has already been started!')}))
		    if ret == "Busy":
                        raise HTTPResponse(status=200, header={'Cache-Control': 'no-store'}, output=json.dumps({'action': False, 'error': _('System is busy')}))


@app.hook('after_request')
def ajax_no_cache():
    # if this is an ajax request
    if request.path.startswith('/api/'):
        response.headers['Cache-Control'] = 'no-store'


# Add authentications
from beaker.middleware import SessionMiddleware

# Add cookie-based session support
session_opts = {
    'session.type': 'file',
    'session.cookie_expires': True,
    'session.timeout': 900,
    'session.data_dir': './session',
    'session.auto': True,
    'session.save_accessed_time': True,
    'session.encrypt_key': '98763587hkhdaf7*(^*(#%&$*YRhhgda',
    'session.validate_key': '98763587hkhdaf7*(^*(#%&$*YRhhgda'}

app = SessionMiddleware(app, session_opts)

# Run the server
rsc_error_thread = rsc_thread.ErrorInfoThread()
rsc_error_thread.start()
#run(app, host='0.0.0.0', port='8088')

# https 运行
ssl_cherry_server = common_utils.init_ssl_cherry_server("0.0.0.0", "8088")
ssl_cherry_server.run(app)
