#!/usr/bin/python
# -*-coding:utf-8-*-
# coding: iso-8859-15


from common_utils import manager
import json
import os
import commands

def get_rsc_meta(rsc_class, rsc_type, rsc_provider=None):
    # item = {'rsc_class':'ocf', 'rsc_type':'IPaddr', 'rsc_provider': 'common'}
    primitive_class = rsc_class
    if rsc_class == "stonith":
        if rsc_provider is None:
            primitive_type = rsc_type
        else:
            primitive_type = rsc_type + "/" + rsc_provider
        primitive_provider = None
    else:
        primitive_type = rsc_type
        primitive_provider = rsc_provider
    meta = manager.get_rsc_meta(primitive_class, primitive_type, primitive_provider)
    if meta is None:
        actions = ["start", "stop", "reload", "monitor", "status", "migrate_from", "migrate_to", "promote", "demote"]
        op_name_store = [{'name': op_name, 'interval': "0", 'timeout': '0'} for op_name in actions]
        return {'action': True,
                'data': {'name': '', 'version': '', 'longdesc': '', 'shortdesc': '', 'parameters': [],
                         'actions': op_name_store}}

    name = meta.name
    version = meta.version

    longdesc = meta.longdesc
    if longdesc is None:
        longdesc = ""
    else:
        longdesc = longdesc.replace(_(".  "), _(".\n"))
    # longdesc = longdesc.replace("'","&apos;")

    shortdesc = meta.shortdesc
    if shortdesc is None:
        shortdesc = ""

    parameters = meta.parameters
    for parameter in parameters:
	if 'name' in parameter and parameter['name'] == 'cmd_prompt':
	    parameter["value"] = ""
	    parameter["content"]["default"] = ""

    actions = meta.actions
    for action in actions:
        # 删除无用的monitor depth属性
        if 'depth' in action:
	    action.pop('depth')
        if "interval" not in action.keys():
            action["interval"] = "0"

    if rsc_class == 'stonith':
        pcmk_host_list = {'content': {'default': "", 'type': "string"},
                          'longdesc': "A list of machines controlled by this device.", 'name': "pcmk_host_list",
                          'required': "1", 'shortdesc': "", 'unique': "", 'value': ""}
        parameters.append(pcmk_host_list)

    retjson = {'action': True,
               'data': {'name': name, 'version': version, 'longdesc': longdesc, 'shortdesc': shortdesc,
                        'parameters': parameters, 'actions': actions}}
    return retjson

def get_rsc_types(cl):
    status, output = commands.getstatusoutput("pcs resource agents " + cl)
    if status == 0:
        ra_list = output.split('\n')
        return ra_list
    else:
	return None

def get_all_rsc_avail():

    ret_class = {}
    classes = manager.get_rsc_classes()
    for cl in classes:
	# 原函数偶尔会发生获取值为None的问题
        #ra_list = manager.get_rsc_types(cl)
        ra_list = get_rsc_types(cl)
	# 有时ra_list数据会获取为空值,添加重连机制
        if ra_list is None:
	    return {'action': False, 'data': _('The System is busy')}
        ra_list.sort()
        provider_list = []
        if cl == "ocf" and ra_list is not None:
            ret_provider = {}
            for ra in ra_list:
                provider_list = manager.get_rsc_providers(cl, ra)
                if provider_list is None:
                    break
                for provider in provider_list:
                    if provider in ret_provider.keys():
			# 判断ocf是否存在重复的类型名称，没有则继续添加
			if ra not in ret_provider[provider]:
                            ret_provider[provider].append(ra)
                    else:
                        ret_provider[provider] = []
                        ret_provider[provider].append(ra)
            ret_class['ocf'] = ret_provider
        elif cl == 'heartbeat':
            continue
        else:
	    ra_list_new = list(set(ra_list)) 
	    if 'colord' in ra_list_new:
	        ra_list_new.remove('colord')
            ret_class[cl] = ra_list_new

    return {'action': True, 'data': ret_class}
